@@ -297,7 +297,7 @@ write_d10_texture <- function(piece_side = "die_face", suit = 1, rank = 1, cfg =
                               ..., filename = tempfile(fileext = ".png"), res = 72) {
 
     current_dev <- grDevices::dev.cur()
-    if (current_dev > 1) on.exit(grDevices::dev.set(current_dev))
+    if (current_dev > 1) on.exit(grDevices::dev.set(current_dev), add = TRUE)
     opt <- cfg$get_piece_opt(piece_side, suit, rank)
     height <- cfg$get_height("die_face", suit, rank)
     width <- cfg$get_width("die_face", suit, rank)
