@@ -10,9 +10,9 @@ test_that("geom_piece() works as expected", {
 
     df_board <- tibble(piece_side = "board_face", suit = 3, rank = 8,
                    x = 4.5, y = 4.5)
-    df_w <- tibble(piece_side = "bit_face", suit = 6, rank = 1,
+    df_w <- tibble(piece_side = "bit_back", suit = 6, rank = 1,
                    x = rep(1:8, 2), y = rep(1:2, each=8))
-    df_b <- tibble(piece_side = "bit_face", suit = 1, rank = 1,
+    df_b <- tibble(piece_side = "bit_back", suit = 1, rank = 1,
                    x = rep(1:8, 2), y = rep(7:8, each=8))
     df <- rbind(df_board, df_w, df_b)
 
